@@ -1,18 +1,13 @@
 <?php
 #-------------------------------------------------------------------------
 # Module: FormBuilder
-# Version: 0.6, released 2008
-#
-# Copyright (c) 2007, Samuel Goldstein <sjg@cmsmodules.com>
-# For Information, Support, Bug Reports, etc, please visit the
-# CMS Made Simple Forge:
-# http://dev.cmsmadesimple.org/projects/formbuilder/
-#
+# Author: Samuel Goldstein, Morten Poulsen
 #-------------------------------------------------------------------------
-# CMS - CMS Made Simple is (c) 2006 by Ted Kulp (wishy@cmsmadesimple.org)
+# CMS Made Simple is (c) 2004 - 2011 by Ted Kulp (wishy@cmsmadesimple.org)
+# CMS Made Simple is (c) 2011 - 2014 by The CMSMS Dev Team
 # This project's homepage is: http://www.cmsmadesimple.org
+# The module's homepage is: http://dev.cmsmadesimple.org/projects/formbuilder
 #-------------------------------------------------------------------------
-#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
@@ -26,8 +21,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 # Or read it online: http://www.gnu.org/licenses/licenses.html#GPL
-#
 #-------------------------------------------------------------------------
+
 class FormBuilder extends CMSModule
 {
 
@@ -37,30 +32,39 @@ class FormBuilder extends CMSModule
 	var $all_validation_types;
 	var $module_ptr;
 	var $module_id;
+	#### JoMorg
+	# deprecated since 0.8.1
 	var $email_regex;
+	# deprecated since 0.8.1
 	var $email_regex_relaxed;
+	#### JoMorg end
+	
 	var $dbHandle;
 
-	function FormBuilder()
+	function __construct()
 	{
-		global $gCms;
-		$this->CMSModule();
-		
+		parent::__construct();
+
 		$this->module_ptr = &$this;
 		$this->dbHandle =  $this->GetDb();
 		$this->module_id = '';
+		#### JoMorg
+		# deprecated since 0.8.1
 		$this->email_regex = "/^([\w\d\.\-\_])+\@([\w\d\.\-\_]+)\.(\w+)$/i";
 		$this->email_regex_relaxed="/^([\w\d\.\-\_])+\@([\w\d\.\-\_])+$/i";
+		#### JoMorg end
+		
 		require_once dirname(__FILE__).'/classes/Form.class.php';
-		require_once dirname(__FILE__).'/classes/FieldBase.class.php';
+		require_once dirname(__FILE__).'/classes/FieldBase.class.php';		
 	}
-
+	
 	function initialize()
 	{
 		$dir=opendir(dirname(__FILE__).'/classes');
 		$this->field_types = array();
 		while($filespec=readdir($dir))
 		{
+			if( !endswith($filespec,'.php') ) continue;
 			if(strpos($filespec,'Field') === false && strpos($filespec,'Disposition') === false)
 			{
 				continue;
@@ -96,80 +100,56 @@ class FormBuilder extends CMSModule
 		ksort($this->std_field_types);
 	}
 
-	function AllowAutoInstall()
-	{
-		return FALSE;
-	}
-
-	function AllowAutoUpgrade()
-	{
-		return FALSE;
-	}
-
-	function GetName()
-	{
-		return 'FormBuilder';
-	}
-	
-	function GetFriendlyName()
-	{
-		return $this->Lang('friendlyname');
-	}
-
-	function GetVersion()
-	{
-		return '0.6';
-	}
-
-	function GetAuthor()
-	{
-		return 'SjG';
-	}
-
-	function GetAuthorEmail()
-	{
-		return 'sjg@cmsmodules.com';
-	}
-
-	function GetAdminDescription($lang = 'en_US')
-	{
-		return $this->Lang('admindesc');
-	}
-
-	function GetChangeLog()
-	{
-		$fn = dirname(__FILE__).'/changelog.inc';
-		return @file_get_contents($fn);
-	}
-
-	function IsPluginModule()
-	{
-		return true;
-	}
-
-	function HasAdmin()
-	{
-		return true;
-	}
-
-	function VisibleToAdminUser()
-	{
-		return $this->CheckPermission('Modify Forms');
-	}
-
+	function AllowAutoInstall() { return FALSE; }
+	function AllowAutoUpgrade() { return FALSE; }
+	function GetName() { return 'FormBuilder'; }
+	function GetFriendlyName() { return $this->Lang('friendlyname'); }
+	function GetVersion() { return '0.8.1.3'; }
+	function GetAuthor() { return 'Morten Poulsen'; }
+	function GetAuthorEmail() { return 'morten@poulsen.org'; }
+	function GetAdminDescription($lang = 'en_US') { return $this->Lang('admindesc'); }
+	function GetChangeLog() { return file_get_contents(dirname(__FILE__).'/changelog.inc'); }
+	function IsPluginModule() { return true; }
+	function HasAdmin() { return true; }
+	function VisibleToAdminUser() { return $this->CheckPermission('Modify Forms'); }
+	function LazyLoadAdmin() { return false; }
 	function AdminStyle()
 	{
-		return "\n.module_fb_table {font-size: 10px;}\n.module_fb_area_wide {width: 500px;}\n.module_fb_legend{font-size: 9px; margin: 6px; border: 1px solid black;}.module_fb_area_short {width: 500px; height: 100px;}\n.module_fb_link {text-decoration: underline;}\n.module_fb_fieldset {margin-bottom:2em;}\n.odd {background-color: #fff;text-align:left;vertical-alignment: top;}\n.even {background-color: #ddd;text-align:left;vertical-alignment: top;}\n";
+		$output = '';
+		
+		$fn = dirname(__FILE__).'/includes/admin.css';
+		$output .= @file_get_contents($fn);
+		
+		return $output;
 	}
 
+	function GetHeaderHTML()
+	{
+		$config = cmsms()->GetConfig();
+		$tmpl = '';
+
+		if( version_compare($GLOBALS['CMS_VERSION'],'1.9') < 0 )
+		{
+		    $tmpl .= '<script type="text/javascript" src="'.$config['root_url'].'/modules/'.$this->GetName().'/includes/jquery-1.4.2.min.js"></script>';
+		}
+		$tmpl .= '<script type="text/javascript" src="'.$config['root_url'].'/modules/'.$this->GetName().'/includes/jquery.tablednd.js"></script>';		
+		$tmpl .= '<script type="text/javascript" src="'.$config['root_url'].'/modules/'.$this->GetName().'/includes/fb_jquery_functions.js"></script>';
+		$tmpl .= '<script type="text/javascript" src="'.$config['root_url'].'/modules/'.$this->GetName().'/includes/fb_jquery.js"></script>';
+		
+        return $this->ProcessTemplateFromData($tmpl);
+	}
+	
 	function SetParameters()
 	{
 		$this->RegisterModulePlugin();
 		$this->RestrictUnknownParams();
+		
 		$this->CreateParameter('fbrp_*','null',$this->Lang('formbuilder_params_general'));
 		$this->SetParameterType(CLEAN_REGEXP.'/fbrp_.*/',CLEAN_STRING);
+		
 		$this->CreateParameter('form_id','null',$this->Lang('formbuilder_params_form_id'));
 		$this->SetParameterType('form_id',CLEAN_INT);
+		
 		$this->CreateParameter('form','null',$this->Lang('formbuilder_params_form_name'));
 		$this->SetParameterType('form',CLEAN_STRING);
 		
@@ -185,26 +165,15 @@ class FormBuilder extends CMSModule
 
 	function DoAction($name,$id,$params,$returnid='')
 	{
+		$smarty = cmsms()->GetSmarty();
+		
 		$this->module_id = $id;
 		parent::DoAction($name,$id,$params,$returnid);
 	}
 
-	function GetDependencies()
-	{
-		return array('CMSMailer'=>'1.73');
-	}
-
-	// may be too stringent, but better safe than sorry.
-	function MinimumCMSVersion()
-	{
-		return '1.1';
-	}
-
-	function InstallPostMessage()
-	{
-		return $this->Lang('post_install');
-	}
-
+	function GetDependencies() { return array('CMSMailer'=>'1.73'); }
+	function MinimumCMSVersion() { return '1.11'; }
+	function InstallPostMessage() { return $this->Lang('post_install'); }
 
 	function CheckAccess($permission='Modify Forms')
 	{
@@ -221,6 +190,7 @@ class FormBuilder extends CMSModule
 	database access. If we let ADODB quote it, the SQL is not valid (not that MySQL cares,
 	but Postgres does).
 	*/
+	
 	function GetForms($order_by='name')
 	{
 		$db = $this->GetDb();
@@ -270,12 +240,14 @@ class FormBuilder extends CMSModule
 		return new fbForm($this, $params, $loadDeep);
 	}
 
-	
-	function GetHelp($lang = 'en_US')
+	function GetHelp() 
 	{
-		return $this->Lang('help');
+		$smarty = cmsms()->GetSmarty();
+		$smarty->assign('mod', $this);
+		
+		return $this->ProcessTemplate('help_text.tpl'); 
 	}
-
+	
 	function GetResponse($form_id,$response_id,$field_list=array(), $dateFmt='d F y')
 	{
 		$names = array();
@@ -283,13 +255,13 @@ class FormBuilder extends CMSModule
 		$db = $this->GetDb();
 		$fbField = $this->GetFormBrowserField($form_id);
 		if ($fbField == false)
-			{
+		{
 			// error handling goes here.
 			echo($this->Lang('error_has_no_fb_field'));
-			}
+		}
 
 		$dbresult = $db->Execute('SELECT * FROM '.cms_db_prefix().
-					'module_fb_formbrowser WHERE fbr_id=?', array($response_id));
+			'module_fb_formbrowser WHERE fbr_id=?', array($response_id));
 
 		$oneset = new stdClass();
 		if ($dbresult && $row = $dbresult->FetchRow())
@@ -298,15 +270,19 @@ class FormBuilder extends CMSModule
 			$oneset->user_approved = (empty($row['user_approved'])?'':date($dateFmt,$db->UnixTimeStamp($row['user_approved']))); 
 			$oneset->admin_approved = (empty($row['admin_approved'])?'':date($dateFmt,$db->UnixTimeStamp($row['admin_approved'])));
 			$oneset->submitted = date($dateFmt,$db->UnixTimeStamp($row['submitted']));
+			$oneset->user_approved_date = (empty($row['user_approved'])?'':$db->UnixTimeStamp($row['user_approved'])); 
+			$oneset->admin_approved_date = (empty($row['admin_approved'])?'':$db->UnixTimeStamp($row['admin_approved']));
+			$oneset->submitted_date = $db->UnixTimeStamp($row['submitted']);
 			$oneset->xml = $row['response'];
 			$oneset->fields = array();
 			$oneset->names = array();
 			$oneset->fieldsbyalias = array();
 		}
-		
+
 		$populate_names = true;
 		$this->HandleResponseFromXML($fbField, $oneset);
 		list($fnames, $aliases, $vals) = $this->ParseResponseXML($oneset->xml);
+
 		foreach ($fnames as $id=>$name)
 			{
 			if (isset($field_list[$id]) && $field_list[$id] > -1)
@@ -318,7 +294,7 @@ class FormBuilder extends CMSModule
 				{
 				$oneset->fieldsbyalias[$aliases[$id]] = $vals[$id];
 				}
-			}			
+			}
 		return $oneset;
 	}
 
@@ -370,6 +346,18 @@ class FormBuilder extends CMSModule
 		return array($names, $aliases, $vals);
 	}
 
+	// New function as part of fix for Bug 5702 from Mike Hughesdon.
+	function ParseResponseXMLType($xmlstr)
+	{
+		$types = array();
+		$xml = new SimpleXMLElement($xmlstr);
+		foreach ($xml->field as $xmlfield)
+		{
+			$id = (int)$xmlfield['id'];
+			$types['fbrp__'.$id] = (string)$xmlfield['type'];
+		}
+		return $types;
+	}
 
 	function GetFormBrowserField($form_id)
 	{
@@ -439,6 +427,7 @@ class FormBuilder extends CMSModule
 		$values = array();
 		$sql = 'FROM '.cms_db_prefix().
 				'module_fb_formbrowser WHERE form_id=?';
+		$sqlparms = array($form_id);
 		if ($user_approved)
 		{
 			$sql .= ' and user_approved is not null';
@@ -451,22 +440,28 @@ class FormBuilder extends CMSModule
 		{
 			$sql .= ' AND resp_id IN ('. implode(',', $params['fbrp_response_search']) .')';
 		}
+		if (isset($params['filter_field']) && substr($params['filter_field'],0,5) =='index')
+			{
+			$idxfld = intval(substr($params['filter_field'],5));
+			$sql .= ' AND index_key_'.$idxfld.'=?';
+			array_push($sqlparms,$params['filter_value']);
+			}
 		if (! isset($params['fbrp_sort_field']) || $params['fbrp_sort_field']=='submitdate' || empty($params['fbrp_sort_field']))
 		{
-			if (isset($params['fbrp_sort_dir']) && $params['fbrp_sort_dir'] == 'd')
+			if (isset($params['fbrp_sort_dir']) && $params['fbrp_sort_dir'] == 'a')
 			{
-				$sql .= ' order by submitted desc';	
+				$sql .= ' order by submitted asc';
 			}
 			else
 			{
-				$sql .= ' order by submitted asc';
+				$sql .= ' order by submitted desc';
 			}
 		}
 		else if (isset($params['fbrp_sort_field']))
 			{
 			if (isset($params['fbrp_sort_dir']) && $params['fbrp_sort_dir'] == 'd')
 				{
-					$sql .= ' order by index_key_'.(int)$params['fbrp_sort_field'].' desc';	
+					$sql .= ' order by index_key_'.(int)$params['fbrp_sort_field'].' desc';
 				}
 				else
 				{
@@ -474,7 +469,7 @@ class FormBuilder extends CMSModule
 				}
 			}
 
-		$dbcount = $db->Execute('SELECT COUNT(*) as num '.$sql,array($form_id));
+		$dbcount = $db->Execute('SELECT COUNT(*) as num '.$sql,$sqlparms);
 
 		$records = 0;
 		if ($dbcount && $row = $dbcount->FetchRow())
@@ -484,12 +479,13 @@ class FormBuilder extends CMSModule
 
 		if ($number > -1)
 			{
-			$dbresult = $db->SelectLimit('SELECT * '.$sql, $number, $start_point, array($form_id));
+			$dbresult = $db->SelectLimit('SELECT * '.$sql, $number, $start_point, $sqlparms);
 			}
 		else
 			{
-			$dbresult = $db->Execute('SELECT * '.$sql, array($form_id));	
+			$dbresult = $db->Execute('SELECT * '.$sql, $sqlparms);
 			}
+			
 		while ($dbresult && $row = $dbresult->FetchRow())
 		{
 			$oneset = new stdClass();
@@ -497,6 +493,10 @@ class FormBuilder extends CMSModule
 			$oneset->user_approved = (empty($row['user_approved'])?'':date($dateFmt,$db->UnixTimeStamp($row['user_approved']))); 
 			$oneset->admin_approved = (empty($row['admin_approved'])?'':date($dateFmt,$db->UnixTimeStamp($row['admin_approved']))); 
 			$oneset->submitted = date($dateFmt,$db->UnixTimeStamp($row['submitted']));
+			$oneset->user_approved_date = (empty($row['user_approved'])?'':$db->UnixTimeStamp($row['user_approved'])); 
+			$oneset->admin_approved_date = (empty($row['admin_approved'])?'':$db->UnixTimeStamp($row['admin_approved'])); 
+			$oneset->submitted_date = $db->UnixTimeStamp($row['submitted']);
+
 			$oneset->xml = $row['response'];
 			$oneset->fields = array();
 			$oneset->fieldsbyalias = array();
@@ -550,6 +550,97 @@ class FormBuilder extends CMSModule
 		return array($records, $names, $values);
 	}
 
+	// writes all records into a flat file.
+	function WriteSortedResponsesToFile($form_id, $filespec, $striptags=true, $dateFmt='d F y', &$params)
+	{
+		$db = $this->GetDb();
+		$names = array();
+		$values = array();
+		$sql = 'FROM '.cms_db_prefix().
+				'module_fb_formbrowser WHERE form_id=?';
+
+		if (! isset($params['fbrp_sort_field']) || $params['fbrp_sort_field']=='submitdate' || empty($params['fbrp_sort_field']))
+		{
+			if (isset($params['fbrp_sort_dir']) && $params['fbrp_sort_dir'] == 'd')
+			{
+				$sql .= ' order by submitted desc';	
+			}
+			else
+			{
+				$sql .= ' order by submitted asc';
+			}
+		}
+		else if (isset($params['fbrp_sort_field']))
+			{
+			if (isset($params['fbrp_sort_dir']) && $params['fbrp_sort_dir'] == 'd')
+				{
+					$sql .= ' order by index_key_'.(int)$params['fbrp_sort_field'].' desc';	
+				}
+				else
+				{
+					$sql .= ' order by index_key_'.(int)$params['fbrp_sort_field'].' asc';
+				}
+			}
+
+		$fbField = $this->GetFormBrowserField($form_id);
+		if ($fbField == false)
+			{
+			// error handling goes here.
+			echo($this->Lang('error_has_no_fb_field'));
+			}
+
+		$fh = fopen($filespec, 'w+');
+		if ($fh === false)
+			{
+			return false;
+			}
+
+		$dbresult = $db->Execute('SELECT * '.$sql, array($form_id));
+
+		$populate_names = true;
+		while ($dbresult && $row = $dbresult->FetchRow())
+		{
+			$oneset = new stdClass();
+			$oneset->id = $row['fbr_id'];
+			$oneset->user_approved = (empty($row['user_approved'])?'':date($dateFmt,$db->UnixTimeStamp($row['user_approved']))); 
+			$oneset->admin_approved = (empty($row['admin_approved'])?'':date($dateFmt,$db->UnixTimeStamp($row['admin_approved']))); 
+			$oneset->submitted = date($dateFmt,$db->UnixTimeStamp($row['submitted']));
+			$oneset->xml = $row['response'];
+			$this->HandleResponseFromXML($fbField, $oneset);
+			list($fnames, $aliases, $vals) = $this->ParseResponseXML($oneset->xml);
+			if ($populate_names)
+				{
+				if ($striptags)
+		         	{
+					foreach ($fnames as $id=>$name)
+						{
+		            	$fnames[$i] = strip_tags($fnames[$i]);
+		            	}
+		         	}
+				fputs ($fh, $this->Lang('title_submit_date')."\t".
+					$this->Lang('title_approval_date')."\t".
+					$this->Lang('title_user_approved')."\t".
+					implode("\t",$fnames)."\n");
+				$populate_names = false;
+				}
+			fputs ($fh,$oneset->submitted . "\t");
+			fputs ($fh,$oneset->admin_approved . "\t");
+			fputs ($fh,$oneset->user_approved . "\t");
+			foreach ($vals as $tv)
+				{
+				if ($striptags)
+	               {
+	               $tv = strip_tags($tv);
+	               }
+				fputs ($fh,preg_replace('/[\n\t\r]/',' ',$tv));
+				fputs ($fh,"\t");
+				}
+			fputs($fh,"\n");
+		}
+		fclose($fh);
+		return true;
+	}
+
 	function GetSortableFields($form_id)
 	{
 		$ret = array();
@@ -559,12 +650,12 @@ class FormBuilder extends CMSModule
 		if ($fbField == false)
 			{
 			// error handling goes here.
-			return $ret;	
+			return $ret;
 			}
 		return $fbField->getSortFieldList();
 	}
 
-  function getFEUIDFromResponseID($response_id)
+  function GetFEUIDFromResponseID($response_id)
 	{
 	$db = $this->GetDb();
     $sql = 'SELECT feuid FROM ' . cms_db_prefix().
@@ -576,11 +667,18 @@ class FormBuilder extends CMSModule
     return -1;
 	}
 
-  function getResponseIDFromFEUID($feu_id)
+  function GetResponseIDFromFEUID($feu_id, $form_id=-1)
 	{
 	$db = $this->GetDb();
-    $sql = 'SELECT fbr_id FROM ' . cms_db_prefix().
-      'module_fb_formbrowser where feuid=?';
+
+	// Fix for Bug 5422. Adapted from Mike Hughesdon's code.
+	$sql = 'SELECT fbr_id FROM ' . cms_db_prefix().
+			'module_fb_formbrowser where feuid=?';
+	if ($form_id != -1)
+	{
+		$sql .= ' and form_id = '.$form_id.' ORDER BY submitted DESC';
+	}
+
     if($result = $db->GetRow($sql, array($feu_id)))
       {
 	    return $result['fbr_id'];
@@ -667,32 +765,21 @@ class FormBuilder extends CMSModule
 		return false;
 	}
 
-	function ReturnFileLock()
-	{
-		$this->ClearFileLock();
-	}
+	function ReturnFileLock() { $this->ClearFileLock(); }
 
-	function GetEventDescription ( $eventname )
-	{
-		return $this->Lang('event_info_'.$eventname );
-	}
+	function GetEventDescription ( $eventname ) { return $this->Lang('event_info_'.$eventname ); }
 
-	function GetEventHelp ( $eventname )
-	{
-		return $this->Lang('event_help_'.$eventname );
-	}
+	function GetEventHelp ( $eventname ) { return $this->Lang('event_help_'.$eventname ); }
 
 	function CreatePageDropdown($id,$name,$current='',
 								$addtext='',$markdefault =true)
 	{
-      global $gCms;
-		// we get here (hopefully) when the template is changed
-		// in the dropdown.
+		// we get here (hopefully) when the template is changed in the dropdown.
 		$db = $this->GetDb();
 		$defaultid = '';
 		if( $markdefault )
 		{
-			$contentops = $gCms->GetContentOperations();
+			$contentops = cmsms()->GetContentOperations();
 			$defaultid = $contentops->GetDefaultPageID();
 		}
 		
@@ -782,40 +869,17 @@ class FormBuilder extends CMSModule
 	return $val;
 	}
    
-/*   function fbencrypt($string,$key)
-      {
-      $key = substr(md5($key),0,24);
-      $td = mcrypt_module_open ('tripledes', '', 'cbc', '');
-	  $iv = '';
-	  for ($i=0;$i<mcrypt_enc_get_iv_size ($td);$i++)
+	function GetActiveTab(&$params)
 		{
-		$iv	.= chr($i);
-		}
-      mcrypt_generic_init ($td, $key, $iv);
-      $enc = base64_encode(mcrypt_generic ($td, $string));
-      mcrypt_generic_deinit ($td);
-      mcrypt_module_close ($td);
-      return $enc;
-      }
-
-   function fbdecrypt($crypt,$key)
-      {
-      $crypt = base64_decode($crypt);
-      $td = mcrypt_module_open ('tripledes', '', 'cbc', '');
-      $key = substr(md5($key),0,24);
-	  $iv = '';
-	  for ($i=0;$i<mcrypt_enc_get_iv_size ($td);$i++)
-		{
-		$iv	.= chr($i);
-		}
-      mcrypt_generic_init ($td, $key, $iv);
-      $plain = mdecrypt_generic ($td, $crypt);
-      mcrypt_generic_deinit ($td);
-      mcrypt_module_close ($td);
-      return $plain;
-      }
- 
-*/
+		if (FALSE == empty($params['active_tab']))
+			{
+		    return $params['active_tab'];
+		  	}
+		else
+			{
+			return 'maintab';
+			}
+	}
   
    function fbencrypt($string,$key)
       {
@@ -842,7 +906,7 @@ class FormBuilder extends CMSModule
       return $plain;
       }
 
-  function fbCreateInputText($id, $name, $value='', $size='10', $maxlength='255', $addttext='')
+  function fbCreateInputText($id, $name, $value='', $size='10', $maxlength='255', $addttext='', $type='text')
 	{
   	$value = cms_htmlentities($value);
   	$id = cms_htmlentities($id);
@@ -852,7 +916,7 @@ class FormBuilder extends CMSModule
 
   	$value = str_replace('"', '&quot;', $value);
 
-  	$text = '<input type="text" name="'.$id.$name.'" value="'.$value.'" size="'.$size.'" maxlength="'.$maxlength.'"';
+  	$text = '<input type="'.$type.'" name="'.$id.$name.'" value="'.$value.'" size="'.$size.'" maxlength="'.$maxlength.'"';
   	if ($addttext != '')
     	{
       	$text .= ' ' . $addttext;
@@ -863,11 +927,10 @@ class FormBuilder extends CMSModule
 
 	function fbCreateInputSubmit($id, $name, $value='', $addttext='', $image='', $confirmtext='')
 	{
-	  $id = cms_htmlentities($id);
-	  $name = cms_htmlentities($name);
-	  $image = cms_htmlentities($image);
-		global $gCms;
-		$config = $gCms->GetConfig();
+		$id = cms_htmlentities($id);
+		$name = cms_htmlentities($name);
+		$image = cms_htmlentities($image);
+		$config = cmsms()->GetConfig();
 		$text = '<input name="'.$id.$name.'" value="'.$value.'" type=';
 		if ($image != '')
 		{
@@ -892,7 +955,75 @@ class FormBuilder extends CMSModule
 	}
 
 
+  function DeleteFromSearchIndex(&$params)
+	{
+		$aeform = new fbForm($this, $params, true);
+		
+		// find browsers keyed to this
+		$browsers = $aeform->GetFormBrowsersForForm();
+		if (count($browsers) < 1)
+			{
+			return;
+			}
+
+		$module =& $this->module_ptr->GetModuleInstance('Search');
+	    if ($module != FALSE)
+	      {
+			foreach ($browsers as $thisBrowser)
+				{
+				$module->DeleteWords( 'FormBrowser', $params['response_id'], 'sub_'.$thisBrowser);	
+				}
+	      }
+	}
+
+  function RegisterTinyMCEPlugin() {
+    
+    $config=$this->GetConfig();
+    $plugin = "
+	tinymce.create('tinymce.plugins.formpicker', {
+        createControl: function(n, cm) {
+            switch (n) {
+                 case 'formpicker':
+                    var c = cm.createMenuButton('formpicker', {
+                        title : '" . $this->GetFriendlyName() . "',
+                        image : '" . $config["root_url"] . "/modules/FormBuilder/images/info-small.gif',
+                        icons : false
+                        });
+
+                        c.onRenderMenu.add(function(c, m) {
+                ";
+    $forms = $this->GetForms();
+    
+    foreach ($forms as $form) {     
+        $plugin .= "
+            m.add({
+                title : '" . $form['name'] . "',
+                onclick : function() {
+                  tinyMCE.activeEditor.execCommand('mceInsertContent', false, '&#123;FormBuilder form=\"".$form["alias"]."\"&#125;');
+                }
+      });
+      ";
+    }
+
+
+
+    $plugin .= "
+                });
+              // Return the new menu button instance
+              return c;
+            }
+
+          return null;
+       }
+});
+";
+ return array(array('formpicker', $plugin, $this->GetFriendlyName()));
+  }
+
+
 } // End of Class
 
-# vim:ts=4 sw=4 noet
-?>
\ No newline at end of file
+#
+# EOF
+#
+?>
