@@ -7,7 +7,8 @@
   This project's homepage is: http://www.cmsmadesimple.org
 */
 
-class fbFieldBase {
+class fbFieldBase
+{
 
   var $Id=-1;
   var $FormId;
@@ -47,88 +48,93 @@ class fbFieldBase {
   function fbFieldBase(&$form_ptr, &$params)
   {
 
-    $this->form_ptr = $form_ptr;
-    $mod = $form_ptr->module_ptr;
-    $this->Options = array();
-    $this->DisplayInForm = true;
-    $this->DisplayInSubmission = true;
-    $this->IsDisposition = false;
-    $this->ValidationTypes = array($mod->Lang('validation_none')=>'none');
-    $this->loaded = false;
-    $this->NonRequirableField = false;
-    $this->HasAddOp = false;
-    $this->HasDeleteOp = false;
-    $this->HasUserAddOp = false;
-    $this->HasUserDeleteOp = false;
-    $this->modifiesOtherFields = false;
-    $this->hasMultipleFormComponents = false;
-    $this->DispositionPermitted = true;
-    $this->SmartyEval = false;
-    $this->labelSubComponents = true;
-    $this->sortable = true;
-    $this->IsComputedOnSubmission = false;
+	  $this->form_ptr = $form_ptr;
+	  $mod = formbuilder_utils::GetFB();
+	  $this->Options = array();
+	  $this->DisplayInForm = true;
+	  $this->DisplayInSubmission = true;
+	  $this->IsDisposition = false;
+	  $this->IsEmailDisposition = false;
+	  $this->ValidationTypes = array($mod->Lang('validation_none')=>'none');
+	  $this->loaded = false;
+	  $this->NonRequirableField = false;
+	  $this->HasAddOp = false;
+	  $this->HasDeleteOp = false;
+	  $this->HasUserAddOp = false;
+	  $this->HasUserDeleteOp = false;
+	  $this->modifiesOtherFields = false;
+	  $this->hasMultipleFormComponents = false;
+	  $this->DispositionPermitted = true;
+	  $this->SmartyEval = false;
+	  $this->labelSubComponents = true;
+	  $this->sortable = true;
+	  $this->IsComputedOnSubmission = false;
 
-    if (isset($params['form_id']))
-      {
-	$this->FormId = $params['form_id'];
-      }
-    if (isset($params['field_id']))
-      {
-	$this->Id = $params['field_id'];
-      }
-    if (isset($params['fbrp_field_name']))
-      {
-	$this->Name = $params['fbrp_field_name'];
-      }
-    if (isset($params['fbrp_field_type']))
-      {
-	$this->Type = $params['fbrp_field_type'];
-      }
-    else
-      {
-	$this->Type = '';
-      }
-    if (isset($params['fbrp_order_by']))
-      {
-	$this->OrderBy = $params['fbrp_order_by'];
-      }
-    if (isset($params['fbrp_hide_label']))
-      {
-	$this->HideLabel = $params['fbrp_hide_label'];
-      }
-    else if (isset($params['fbrp_set_from_form']))
-      {
-	$this->HideLabel = 0;
-      }
-    if (isset($params['fbrp_required']))
-      {
-	$this->Required = $params['fbrp_required'];
-      }
-    else if (isset($params['fbrp_set_from_form']))
-      {
-	$this->Required = 0;
-      }
-    if (isset($params['fbrp_validation_type']))
-      {
-	$this->ValidationType = $params['fbrp_validation_type'];
-      }
+    if (isset($params['form_id'])) {
+	
+		$this->FormId = $params['form_id'];
+    }
+	
+    if (isset($params['field_id'])) {
+	
+		$this->Id = $params['field_id'];
+    }
+	
+    if (isset($params['fbrp_field_name'])) {
+	
+		$this->Name = $params['fbrp_field_name'];
+    }
+	
+    if (isset($params['fbrp_field_type'])) {
+	
+		$this->Type = $params['fbrp_field_type'];
+    } else {
+	
+		$this->Type = '';
+    }
+	
+    if (isset($params['fbrp_order_by'])) {
 
-    foreach ($params as $thisParamKey=>$thisParamVal)
-      {
-	if (substr($thisParamKey,0,9) == 'fbrp_opt_')
-	  {
-	    $thisParamKey = substr($thisParamKey,9);
-	    $this->Options[$thisParamKey] = $thisParamVal;
-	  }
-      }
+		$this->OrderBy = $params['fbrp_order_by'];
+    }
+	
+    if (isset($params['fbrp_hide_label'])) {
+	
+		$this->HideLabel = $params['fbrp_hide_label'];
+    } elseif (isset($params['fbrp_set_from_form'])) {
+	
+		$this->HideLabel = 0;
+    }
+	
+    if (isset($params['fbrp_required'])) {
+	
+		$this->Required = $params['fbrp_required'];
+    } elseif (isset($params['fbrp_set_from_form'])) {
+	
+		$this->Required = 0;
+    }
+	
+    if (isset($params['fbrp_validation_type'])) {
+	
+		$this->ValidationType = $params['fbrp_validation_type'];
+    }
 
-    if (isset($params['fbrp__'.$this->Id]) &&
-	(is_array($params['fbrp__'.$this->Id]) ||
-	 strlen($params['fbrp__'.$this->Id]) > 0))
-      {
+    foreach ($params as $thisParamKey=>$thisParamVal) {
+	
+		if (substr($thisParamKey,0,9) == 'fbrp_opt_') {
+		
+			$thisParamKey = substr($thisParamKey,9);
+			$this->Options[$thisParamKey] = $thisParamVal;
+		}
+    }
+
+	// Check value setup against $params
+    if (isset($params['fbrp__'.$this->Id]) && (is_array($params['fbrp__'.$this->Id]) || strlen($params['fbrp__'.$this->Id]) > 0)) {
+	
 	   $this->SetValue($params['fbrp__'.$this->Id]);
-      }
-  }
+    }
+		
+  } // end of __construct()
 
   function HasMultipleFormComponents()
   {
@@ -144,8 +150,7 @@ class fbFieldBase {
   {
     return $this->IsComputedOnSubmission;
   }
-
-
+  
   // override me 
   function ComputeOrder()
   {
@@ -206,7 +211,20 @@ class fbFieldBase {
   {
     return '';
   }
+
+  // Sends logic along with field, also allows smarty logic
+  // Dosen't need override in most cases
+  function GetFieldLogic() 
+  {
+	$mod = formbuilder_utils::GetFB();
+	$code = $this->GetOption('field_logic','');
 	
+	if(!empty($code)) {
+	
+		return $mod->ProcessTemplateFromData($code);
+	}
+  }
+  
   // override me with something to show users
   function StatusInfo()
   {
@@ -239,7 +257,7 @@ class fbFieldBase {
   // override me
   function GetOptionAddButton()
   {
-    $mod = $this->form_ptr->module_ptr;
+    $mod = formbuilder_utils::GetFB();
     return $mod->Lang('add_options');
   }
 	
@@ -256,7 +274,7 @@ class fbFieldBase {
   // override me
   function GetOptionDeleteButton()
   {
-    $mod = $this->form_ptr->module_ptr;
+    $mod = formbuilder_utils::GetFB();
     return $mod->Lang('delete_options');
   }
 
@@ -358,6 +376,11 @@ class fbFieldBase {
     return $this->IsDisposition;
   }
 
+  function IsEmailDisposition()
+  {
+    return $this->IsEmailDisposition;
+  }
+
   function HasLabel()
   {
     return $this->HasLabel;
@@ -441,87 +464,83 @@ class fbFieldBase {
   // override me with a displayable type
   function GetDisplayType()
   {
-    return $this->form_ptr->module_ptr->Lang('field_type_'.$this->Type);
+    return formbuilder_utils::GetFB()->Lang('field_type_'.$this->Type);
   }
 
 
+  // Base backended fields configuration
   function PrePopulateBaseAdminForm($formDescriptor,$disposeOnly=0)
   {
-    $mod = $this->form_ptr->module_ptr;
-    if ($this->Type == '')
-      {
-	if ($disposeOnly == 1)
-	  {
-	    $typeInput = $mod->CreateInputDropdown($formDescriptor, 'fbrp_field_type',array_merge(array($mod->Lang('select_type')=>''),$mod->disp_field_types), -1,'', 'onchange="this.form.submit()"');
-	  }
-	else
-	  {
-	    $typeInput = $mod->CreateInputDropdown($formDescriptor, 'fbrp_field_type',array_merge(array($mod->Lang('select_type')=>''),$mod->field_types), -1,'', 'onchange="this.form.submit()"');
-	  }
-      }
-    else
-      {
-	$typeInput = $this->GetDisplayType().$mod->CreateInputHidden($formDescriptor, 'fbrp_field_type', $this->Type);
-      }
+    $mod = formbuilder_utils::GetFB();
+	
+	// Do the field type check
+    if ($this->Type == '') {
+		if ($disposeOnly == 1) {
 		
+			$typeInput = $mod->CreateInputDropdown($formDescriptor, 'fbrp_field_type',array_merge(array($mod->Lang('select_type')=>''),$mod->disp_field_types), -1,'', 'onchange="this.form.submit()"');
+		} else {
+		
+			$typeInput = $mod->CreateInputDropdown($formDescriptor, 'fbrp_field_type',array_merge(array($mod->Lang('select_type')=>''),$mod->field_types), -1,'', 'onchange="this.form.submit()"');
+		}
+    } else {
+	
+		$typeInput = $this->GetDisplayType().$mod->CreateInputHidden($formDescriptor, 'fbrp_field_type', $this->Type);
+    }
+		
+	// Init main tab	
     $main = array(
-		  array($mod->Lang('title_field_name'),
-			$mod->CreateInputText($formDescriptor, 'fbrp_field_name', $this->GetName(), 50)),
-		  array($mod->Lang('title_field_type'),$typeInput),
-		  );
-		
+				array($mod->Lang('title_field_name'),$mod->CreateInputText($formDescriptor, 'fbrp_field_name', $this->GetName(), 50)),
+				array($mod->Lang('title_field_type'),$typeInput)
+			);
+
+	// Init advanced tab
     $adv = array();
 
     // if we know our type, we can load up with additional options
-    if ($this->Type != '')
-      {
+    if ($this->Type != '') {
 			
-	// validation types?
-	if (count($this->GetValidationTypes()) > 1)
-	  {
-	    $validInput = $mod->CreateInputDropdown($formDescriptor, 'fbrp_validation_type', $this->GetValidationTypes(), -1, $this->GetValidationType());
-	  }
-	else
-	  {
-	    $validInput = $mod->Lang('automatic');
-	  }
-				
-	if (!$this->IsNonRequirableField())
-	  {
-	    array_push($main, array($mod->Lang('title_field_required'),$mod->CreateInputCheckbox($formDescriptor, 'fbrp_required', 1, $this->IsRequired()).$mod->Lang('title_field_required_long')));
-	  }
-				
-	array_push($main, array($mod->Lang('title_field_validation'),$validInput));
-
-	if( $this->HasLabel == 1 )
-	  {
-	    array_push($adv, array($mod->Lang('title_hide_label'),$mod->CreateInputCheckbox($formDescriptor, 'fbrp_hide_label', 1, $this->HideLabel()).$mod->Lang('title_hide_label_long')));
-	  }
-
-	$alias = $this->GetOption('field_alias','');
-	if ($alias == '')
-		{
-		$alias = 'fld'.$this->GetId();
+		// validation types?
+		if (count($this->GetValidationTypes()) > 1) {
+		
+			$validInput = $mod->CreateInputDropdown($formDescriptor, 'fbrp_validation_type', $this->GetValidationTypes(), -1, $this->GetValidationType());
+		} else {
+		
+			$validInput = $mod->Lang('automatic');
 		}
-	array_push($adv, array($mod->Lang('title_field_alias'),$mod->CreateInputText($formDescriptor, 'fbrp_opt_field_alias', $this->GetOption('field_alias'), 50)));			
+				
+		if (!$this->IsNonRequirableField()) {
+			array_push($main, array($mod->Lang('title_field_required'),$mod->CreateInputCheckbox($formDescriptor, 'fbrp_required', 1, $this->IsRequired()).$mod->Lang('title_field_required_long')));
+		}
+				
+		array_push($main, array($mod->Lang('title_field_validation'),$validInput));
 
-	if ($this->DisplayInForm())
-	  {
-	    array_push($adv,array($mod->Lang('title_field_css_class'),$mod->CreateInputText($formDescriptor, 'fbrp_opt_css_class', $this->GetOption('css_class'), 50)));
-	    array_push($adv,array($mod->Lang('title_field_helptext'),$mod->CreateTextArea(false, $formDescriptor, $this->GetOption('helptext',''),
-					   'fbrp_opt_helptext','module_fb_area_short')));
-		array_push($adv,array($mod->Lang('title_field_javascript'),
-			$mod->CreateTextArea(false, $formDescriptor, $this->GetOption('javascript',''),
-						   'fbrp_opt_javascript','module_fb_area_short').'<br />'.
-			$mod->Lang('title_field_javascript_long')));
-	  }
+		if( $this->HasLabel == 1 ) {
+			
+			array_push($adv, array($mod->Lang('title_hide_label'),$mod->CreateInputCheckbox($formDescriptor, 'fbrp_hide_label', 1, $this->HideLabel()).$mod->Lang('title_hide_label_long')));
+		}
+
+		$alias = $this->GetOption('field_alias','');
+		if ($alias == '') {
+			$alias = 'fld'.$this->GetId();
+		}
+		
+		array_push($adv, array($mod->Lang('title_field_alias'),$mod->CreateInputText($formDescriptor, 'fbrp_opt_field_alias', $this->GetOption('field_alias'), 50)));			
+
+		if ($this->DisplayInForm()) {
+			array_push($adv,array($mod->Lang('title_field_css_class'),$mod->CreateInputText($formDescriptor, 'fbrp_opt_css_class', $this->GetOption('css_class'), 50)));
+			array_push($adv,array($mod->Lang('title_field_helptext'),$mod->CreateTextArea(false, $formDescriptor, $this->GetOption('helptext',''), 'fbrp_opt_helptext','module_fb_area_short')));
+			array_push($adv,array($mod->Lang('title_field_javascript'),$mod->CreateTextArea(false, $formDescriptor, $this->GetOption('javascript',''), 
+								'fbrp_opt_javascript','module_fb_area_short','','', '', '80', '15','','js').'<br />'.$mod->Lang('title_field_javascript_long')));
+			array_push($adv,array($mod->Lang('title_field_logic'),$mod->CreateTextArea(false, $formDescriptor, $this->GetOption('field_logic',''), 
+								'fbrp_opt_field_logic','module_fb_area_short','','', '', '80', '15').'<br />'.$mod->Lang('title_field_logic_long')));
+			array_push($adv,array($mod->Lang('title_html5'),$mod->CreateInputHidden($formDescriptor,'fbrp_opt_html5','0').$mod->CreateInputCheckbox($formDescriptor, 'fbrp_opt_html5','1',$this->GetOption('html5','0'))));
+		}
 	
-      }
-    else
-      {
-	// no advanced options until we know our type
-	array_push($adv,array($mod->Lang('tab_advanced'),$mod->Lang('notice_select_type')));
-      }
+    } else {
+	
+		// no advanced options until we know our type
+		array_push($adv,array($mod->Lang('tab_advanced'),$mod->Lang('notice_select_type')));
+    }
 				
     return array('main'=>$main, 'adv'=>$adv);
   }
@@ -552,6 +571,12 @@ class fbFieldBase {
   {
   }
 
+  // override me as necessary
+  function PostFieldSaveProcess(&$params)
+  {
+  }
+
+
   // new method as of Oct 09 that should have been here all along...
   function RemoveAdminField(&$array, $fieldname)
   {
@@ -581,18 +606,25 @@ class fbFieldBase {
   // clear fields unused by invisible dispositions
   function HiddenDispositionFields(&$mainArray, &$advArray, $hideReq=true)
   {
-    $mod = $this->form_ptr->module_ptr;
-    if ($hideReq)
-    	{
-    	// remove the "required" field
+	$mod = formbuilder_utils::GetFB();
+
+	// remove the "required" field
+    if ($hideReq) {
       $this->RemoveAdminField($mainArray, $mod->Lang('title_field_required'));
-     	}
+    }
+	
     // remove the "hide name" field
     $this->RemoveAdminField($advArray, $mod->Lang('title_hide_label'));
+	
     // remove the "css" field
     $this->RemoveAdminField($advArray, $mod->Lang('title_field_css_class'));
+	
     // hide "javascript"
     $this->RemoveAdminField($advArray, $mod->Lang('title_field_javascript'));
+	
+    // hide "logic"
+    $this->RemoveAdminField($advArray, $mod->Lang('title_field_logic'));	
+	
     // hide "help text"
     $this->RemoveAdminField($advArray, $mod->Lang('title_field_helptext'));
 
@@ -613,7 +645,7 @@ class fbFieldBase {
   // override me!
   function GetHumanReadableValue($as_string=true)
   {
-    $mod = $this->form_ptr->module_ptr;
+    $mod = formbuilder_utils::GetFB();
     if ($this->Value !== false)
       {
 		$ret = $this->Value;
@@ -631,19 +663,57 @@ class fbFieldBase {
 		return array($ret);
 		}
   }
+  
+  // override me!
+  function GetAllHumanReadableValues()
+  {
+		
+	$mod = formbuilder_utils::GetFB();	
+	if(in_array('option_value',$this->GetOptionNames())) {
+		
+		if(count($this->GetOption('option_value')) > 0) {
+		
+			return $this->GetOption('option_value');
+		}
+	}
+	
+	return false;
+  }	
 
 
   // override this if you have some unusual format for values,
   // especially if "false" is a valid value!
-  function HasValue()
+  function HasValue($deny_blank_responses=false)
   {
-     $def = $this->GetOption('default','');
-     if ($this->Value !== false &&
-      ($def == '' || $this->Value != $def))
+    if ($this->GetFieldType() == 'TextField' || $this->GetFieldType() == 'TextAreaField')
+    {
+      // fields with defaults
+      $def = $this->GetOption('default', '');
+      
+      if( $this->Value !== false && ($def == '' || $this->Value != $def) )
       {
+        if( $deny_blank_responses && !is_array($this->Value) && preg_match('/^\s+$/',$this->Value) )
+        {
+          return false;
+        }
+        
       return true;
       }
-   return false;
+    }
+    #elseif ($this->Value !== false && ( is_array($this->Value) && strlen( implode('', $this->Value) ) > 0))
+    # ^ This wouldn't work: a value can't simultaneously be and not be an array... (JM)
+    # so no fix to bug #7371 yet (JM)
+    elseif ($this->Value !== false) 
+    {
+      if ($deny_blank_responses && (!is_array($this->Value) && preg_match('/^\s+$/', $this->Value)))
+      {
+        return false;
+      }
+      
+      return true;
+    }
+
+    return false;
   }
 	
   // probably don't need to override this
@@ -700,45 +770,43 @@ class fbFieldBase {
 
   //error_log($this->GetName().':'.print_r($valStr,true));
     $fm = $this->form_ptr;
+    
     if ($this->Value === false)
+    {
+      if ( is_array($valStr) )
       {
-      if (is_array($valStr))
-         {
          $this->Value = $valStr;
-         for ($i=0;$i<count($this->Value);$i++)
-            {
+         
+         for ($i = 0; $i < count( $this->Value ) ; $i++)
+         {
             while ( $this->Value[$i] != $fm->unmy_htmlentities($this->Value[$i]))
-               {
+            {
                $this->Value[$i] = $fm->unmy_htmlentities($this->Value[$i]);
-               }
             }
          }
+      }
       else
-         {
-         while ($this->Value != $fm->unmy_htmlentities($valStr))
-            {
-	         $this->Value = $fm->unmy_htmlentities($valStr);
-	         }
-	      }
-      }
-    else
       {
-      while ($valStr != $fm->unmy_htmlentities($valStr))
+         while ($this->Value !== $fm->unmy_htmlentities($valStr))
          {
+	         $this->Value = $fm->unmy_htmlentities($valStr);
+	       }
+	    }
+    }
+    else
+    {
+      while ($valStr != $fm->unmy_htmlentities($valStr))
+      {
          $valStr = $fm->unmy_htmlentities($valStr);
-         }
-	if (! is_array($this->Value))
-	  {
-	    $this->Value = array($this->Value);
-	  }
-	array_push($this->Value,$valStr);
       }
-  }
-
-  // override me, as necessary
-  function CompareTo($val)
-  {
-    return strcmp($val->GetHumanReadableValue(), $this->GetHumanReadableValue());
+      
+			if (! is_array($this->Value))
+		  {
+		    $this->Value = array($this->Value);
+		  }
+		  
+				array_push($this->Value,$valStr);
+    }
   }
 
   function RequiresValidation()
@@ -755,7 +823,7 @@ class fbFieldBase {
 
   function DoesFieldNameExist()
   {
-    $mod = $this->form_ptr->module_ptr;
+    $mod = formbuilder_utils::GetFB();
 		
     // field name in use??
     if ($mod->GetPreference('unique_fieldnames','1') == '1' &&
@@ -770,7 +838,7 @@ class fbFieldBase {
 
    function DoesFieldHaveName()
    {
-    $mod = $this->form_ptr->module_ptr;
+    $mod = formbuilder_utils::GetFB();
   	if ($mod->GetPreference('require_fieldnames','1') == '1' &&
   		strlen($this->GetName()) < 1)
   		{
@@ -788,6 +856,7 @@ class fbFieldBase {
 		{
 		list($ret, $message) = $this->DoesFieldNameExist();
 		}
+		
 	return array($ret, $message);
   }
 
@@ -798,7 +867,7 @@ class fbFieldBase {
   // it returns an array, where the first element is true on success,
   // or false on failure, and the second element is explanatory
   // text for the failure
-  function DisposeForm()
+  function DisposeForm($returnid)
   {
     return array(true, '');
   }	
@@ -1018,7 +1087,7 @@ class fbFieldBase {
   function Load($id, &$params, $loadDeep=false)
   {
     $sql = 'SELECT * FROM ' . cms_db_prefix() . 'module_fb_field WHERE field_id=?';
-    if($result = $this->form_ptr->module_ptr->dbHandle->GetRow($sql, array($this->Id)))
+    if($result = formbuilder_utils::GetFB()->dbHandle->GetRow($sql, array($this->Id)))
       {
 	if (strlen($this->Name) < 1)
 	  {
@@ -1048,7 +1117,7 @@ class fbFieldBase {
       {
 	$sql = 'SELECT name, value FROM ' . cms_db_prefix() .
 	  'module_fb_field_opt WHERE field_id=? ORDER BY option_id';
-	$rs = $this->form_ptr->module_ptr->dbHandle->Execute($sql,
+	$rs = formbuilder_utils::GetFB()->dbHandle->Execute($sql,
 							     array($this->Id));
 	$tmpOpts = array();
 	while ($rs && $results = $rs->FetchRow())
@@ -1089,7 +1158,7 @@ class fbFieldBase {
 
   function Store($storeDeep=false)
   {
-    $mod =  $this->form_ptr->module_ptr;
+    $mod =  formbuilder_utils::GetFB();
     if ($this->Id == -1)
       {
 	$this->Id = $mod->dbHandle->GenID(cms_db_prefix().'module_fb_field_seq');
@@ -1151,10 +1220,10 @@ class fbFieldBase {
 	return false;
       }
     $sql = 'DELETE FROM ' . cms_db_prefix() . 'module_fb_field where field_id=?';
-    $res = $this->form_ptr->module_ptr->dbHandle->Execute($sql,
+    $res = formbuilder_utils::GetFB()->dbHandle->Execute($sql,
 							  array($this->Id));
     $sql = 'DELETE FROM ' . cms_db_prefix() . 'module_fb_field_opt where field_id=?';
-    $res = $this->form_ptr->module_ptr->dbHandle->Execute($sql,
+    $res = formbuilder_utils::GetFB()->dbHandle->Execute($sql,
 							  array($this->Id));
     return true;
   }
